#!/usr/bin/env python

import numpy as np
import wfdb
import math
import persim
from persim import persistent_entropy
from scipy.signal import butter, lfilter
from scipy import stats
from scipy.stats import skew, kurtosis
from wfdb import processing
from ripser import ripser
from scipy.io import loadmat 

#This is the function that extracts the RR Features
def detect_peaks(ecg_measurements,signal_frequency,gain):

        """
        Method responsible for extracting peaks from loaded ECG measurements data through measurements processing.

        This implementation of a QRS Complex Detector is by no means a certified medical tool and should not be used in health monitoring. 
        It was created and used for experimental purposes in psychophysiology and psychology.
        You can find more information in module documentation:
        https://github.com/c-labpl/qrs_detector
        If you use these modules in a research project, please consider citing it:
        https://zenodo.org/record/583770
        If you use these modules in any other project, please refer to MIT open-source license.

        If you have any question on the implementation, please refer to:

        Michal Sznajder (Jagiellonian University) - technical contact (msznajder@gmail.com)
        Marta lukowska (Jagiellonian University)
        Janko Slavic peak detection algorithm and implementation.
        https://github.com/c-labpl/qrs_detector
        https://github.com/jankoslavic/py-tools/tree/master/findpeaks
        
        MIT License
        Copyright (c) 2017 Michal Sznajder, Marta Lukowska
    
        Permission is hereby granted, free of charge, to any person obtaining a copy
        of this software and associated documentation files (the "Software"), to deal
        in the Software without restriction, including without limitation the rights
        to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
        copies of the Software, and to permit persons to whom the Software is
        furnished to do so, subject to the following conditions:
        The above copyright notice and this permission notice shall be included in all
        copies or substantial portions of the Software.
        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
        IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
        FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
        AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
        LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
        OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
        SOFTWARE.

        """


        filter_lowcut = 0.001
        filter_highcut = 15.0
        filter_order = 1
        integration_window = 30  # Change proportionally when adjusting frequency (in samples).
        findpeaks_limit = 0.35
        findpeaks_spacing = 100  # Change proportionally when adjusting frequency (in samples).
        refractory_period = 240  # Change proportionally when adjusting frequency (in samples).
        qrs_peak_filtering_factor = 0.125
        noise_peak_filtering_factor = 0.125
        qrs_noise_diff_weight = 0.25


        # Detection results.
        qrs_peaks_indices = np.array([], dtype=int)
        noise_peaks_indices = np.array([], dtype=int)


        # Measurements filtering - 0-15 Hz band pass filter.
        filtered_ecg_measurements = bandpass_filter(ecg_measurements, lowcut=filter_lowcut, highcut=filter_highcut, signal_freq=signal_frequency, filter_order=filter_order)

        filtered_ecg_measurements[:5] = filtered_ecg_measurements[5]

        # Derivative - provides QRS slope information.
        differentiated_ecg_measurements = np.ediff1d(filtered_ecg_measurements)

        # Squaring - intensifies values received in derivative.
        squared_ecg_measurements = differentiated_ecg_measurements ** 2

        # Moving-window integration.
        integrated_ecg_measurements = np.convolve(squared_ecg_measurements, np.ones(integration_window)/integration_window)

        # Fiducial mark - peak detection on integrated measurements.
        detected_peaks_indices = findpeaks(data=integrated_ecg_measurements,
                                                     limit=findpeaks_limit,
                                                     spacing=findpeaks_spacing)

        detected_peaks_values = integrated_ecg_measurements[detected_peaks_indices]

        return detected_peaks_values,detected_peaks_indices

 

def bandpass_filter(data, lowcut, highcut, signal_freq, filter_order):
        """
        Method responsible for creating and applying Butterworth filter.
        :param deque data: raw data
        :param float lowcut: filter lowcut frequency value
        :param float highcut: filter highcut frequency value
        :param int signal_freq: signal frequency in samples per second (Hz)
        :param int filter_order: filter order
        :return array: filtered data
        """
        nyquist_freq = 0.5 * signal_freq
        low = lowcut / nyquist_freq
        high = highcut / nyquist_freq
        b, a = butter(filter_order, [low, high], btype="band")
        y = lfilter(b, a, data)
        return y

def findpeaks(data, spacing=1, limit=None):
        """
        Janko Slavic peak detection algorithm and implementation.
        https://github.com/jankoslavic/py-tools/tree/master/findpeaks
        Finds peaks in `data` which are of `spacing` width and >=`limit`.
        :param ndarray data: data
        :param float spacing: minimum spacing to the next peak (should be 1 or more)
        :param float limit: peaks should have value greater or equal
        :return array: detected peaks indexes array
        """
        len = data.size
        x = np.zeros(len + 2 * spacing)
        x[:spacing] = data[0] - 1.e-6
        x[-spacing:] = data[-1] - 1.e-6
        x[spacing:spacing + len] = data
        peak_candidate = np.zeros(len)
        peak_candidate[:] = True
        for s in range(spacing):
            start = spacing - s - 1
            h_b = x[start: start + len]  # before
            start = spacing
            h_c = x[start: start + len]  # central
            start = spacing + s + 1
            h_a = x[start: start + len]  # after
            peak_candidate = np.logical_and(peak_candidate, np.logical_and(h_c > h_b, h_c > h_a))

        ind = np.argwhere(peak_candidate)
        ind = ind.reshape(ind.size)
        if limit is not None:
            ind = ind[data[ind] > limit]
        return ind

#This is the barcode-based TDA feature extractor. The inputs are a point-cloud and the maximum dimension of features to be computed and the output is a numpy array of stats-based features extracted from the barcodes.

def extract(diagrams,maxdimension):
    
    #initialize some local variables
    N1 = N2 = MeanBC1 = MeanDC1 = SUMpersC1 = SDBC1 = SDDC1 = SDPersC1 = SKPersC1 = SUMpersC1 = SKBC1 = KurtBC1 = 0
    SKDC1 = KurtDC1 = MeanpersC1 = KurtpersC1 = MeanBC2 = MeanDC2 = SUMpersC2 = SDBC2 = SDDC2 = SDPersC2 = SKPersC2 = 0
    SUMpersC2 = SKBC2 = KurtBC2 = SKDC2 = KurtDC2 = MeanpersC2 = KurtpersC2 = 0
    
    #Start filling in variables
    N0 = len(diagrams[0])  
    C0d = np.array(diagrams[0][:len(diagrams[0])-1,1])
    if len(C0d)>1:        
        MeanDC0 = np.mean(C0d)
        SUMpersC0 = sum(C0d)
        SDDC0 = np.std(C0d)
        SKDC0 = skew(C0d)
        KurtDC0 = kurtosis(C0d)
    elif len(C0d)==1:
        MeanDC0 = np.mean(C0d)
        SUMpersC0 = sum(C0d)
        SDDC0 = 0
        SKDC0 = 0
        KurtDC0 = 0
    else:
        MeanDC0 = 0
        SUMpersC0 = 0
        SDDC0 = 0
        SKDC0 = 0
        KurtDC0 = 0
    d0 = diagrams[0][:len(diagrams[0])-1,1]
    hd0 = np.histogram(d0,bins = 'auto')
    d0 = d0[:len(d0)-1-len(np.where(d0>=hd0[1][len(hd0[1])-2])[0])]
    if len(d0)>1:
        MeanDC0noPers = np.mean(d0)
        SDDC0noPers = np.std(d0)
        SKDC0noPers = skew(d0)
        KurtDC0noPers = kurtosis(d0)
    elif len(d0)==1:
        MeanDC0noPers = d0
        SDDC0noPers = 0
        SKDC0noPers = 0
        KurtDC0noPers = 0
    else:
        MeanDC0noPers = 0
        SDDC0noPers = 0
        SKDC0noPers = 0
        KurtDC0noPers = 0
    MeanBC1noPers = SDBC1noPers = SKBC1noPers = KurtBC1noPers = MeanDC1noPers = SDDC1noPers = SKDC1noPers = KurtDC1noPers = 0
    SUMPersC1noPers = MeanPersC1noPers = SDPersC1noPers = SKPersC1noPers = KurtPersC1noPers = 0
    MeanBC2noPers = SDBC2noPers = SKBC2noPers = KurtBC2noPers = MeanDC2noPers = SDDC2noPers = SKDC2noPers = KurtDC2noPers = 0
    SUMPersC2noPers = MeanPersC2noPers = SDPersC2noPers = SKPersC2noPers = KurtPersC2noPers = 0
    #new_d = 0
    #new_d2 = 0
    pers_entropy = [0,0]
 
    if(len(diagrams[1])>0):
        pers_entropy = []
        for i in range(maxdimension+1):
            pers_entropy = np.append(pers_entropy,persim.persistent_entropy.persistent_entropy(diagrams[i]))

    if(len(diagrams[1])>0):
        N1 = len(diagrams[1])
   
        #print(N1)
        if(N1>1):
            d0=diagrams[1][:len(diagrams[1])-1,0]
            d1=diagrams[1][:len(diagrams[1])-1,1]
            d = np.array(d1-d0)
            #cons = np.array(d[1:])-np.array(d[:len(d)-1])
            #mark = np.argmax(cons)+1
            #diffs = d[mark] - d[:mark]
            #diffs_ave = np.average(diffs)
            C1b = np.array(diagrams[1][:,0])
            C1d = np.array(diagrams[1][:,1])
            MeanBC1 = np.mean(C1b)
            MeanDC1 = np.mean(C1d)
            SUMpersC1 = sum(C1d-C1b)
            SDBC1 = np.std(C1b)
            SDDC1 = np.std(C1d)
            SDPersC1 = np.std(C1d-C1b)
            SKPersC1 = skew(C1d-C1b)
            SKBC1 = skew(C1b)
            KurtBC1 = kurtosis(C1b)
            SKDC1 = skew(C1d)
            KurtDC1 = kurtosis(C1d)
            MeanpersC1 = np.mean(C1d-C1b)
            KurtpersC1 = kurtosis(C1d-C1b)
            hd = np.histogram(d,bins = 'auto')
            d = d[:len(d)-1-len(np.where(d>=hd[1][len(hd[1])-2])[0])]
            MeanBC1noPers = np.mean(d0)
            SDBC1noPers = np.std(d0)
            SKBC1noPers = skew(d0)
            KurtBC1noPers = kurtosis(d0)
            MeanDC1noPers = np.mean(d1)
            SDDC1noPers = np.std(d1)
            SKDC1noPers = skew(d1)
            KurtDC1noPers = kurtosis(d1)
            SUMPersC1noPers = sum(d)
            MeanPersC1noPers = np.mean(d)
            SDPersC1noPers = np.std(d)
            SKPersC1noPers = skew(d)
            KurtPersC1noPers = kurtosis(d)
            #if mark < len(d):
            #    new_d = (np.sum(d[d>=d[mark]])+np.sum(d[:mark][diffs<=diffs_ave*0.4]))/d[mark-1]
            #    new_d2 = len(d[d>=d[mark]])+len(d[:mark][diffs<=diffs_ave*0.4])
            #else:
            #    new_d = max(cons)/d[mark-2]
            #    new_d2 = 1
        else:
            MeanBC1 = diagrams[1][:,0]
            MeanDC1 = diagrams[1][:,1]
            SUMpersC1 = diagrams[1][:,1]-diagrams[1][:,0]
            SDBC1 = SDDC1 = SDPersC1 = SKPersC1 = SKBC1 = KurtBC1 = SKDC1 = KurtDC1 = KurtpersC1 = 100
            MeanpersC1 = diagrams[1][:,1]-diagrams[1][:,0]

            MeanBC1noPers = SDBC1noPers = SKBC1noPers = KurtBC1noPers = MeanDC1noPers = SDDC1noPers = SKDC1noPers = 100
            KurtDC1noPers = SUMPersC1noPers = MeanPersC1noPers = SDPersC1noPers = SKPersC1noPers = KurtPersC1noPers = 100
            
    if(len(diagrams)==3):
        N2 = len(diagrams[2])
        d0=diagrams[2][:len(diagrams[2])-1,0]
        d1=diagrams[2][:len(diagrams[2])-1,1]
        d = d1-d0
        if N2>1:

            C2b = np.array(diagrams[2][:,0])
            C2d = np.array(diagrams[2][:,1])
            MeanBC2 = np.mean(C2b)
            MeanDC2 = np.mean(C2d)
            SUMpersC2 = sum(C2d-C2b)
            SDBC2 = np.std(C2b)
            SDDC2 = np.std(C2d)
            SDPersC2 = np.std(C2d-C2b)
            SKPersC2 = skew(C2d-C2b)
            SKBC2 = skew(C2b)
            KurtBC2 = kurtosis(C2b)
            SKDC2 = skew(C2d)
            KurtDC2 = kurtosis(C2d)
            MeanpersC2 = np.mean(C2d-C2b)
            KurtpersC2 = kurtosis(C2d-C2b)

            hd = np.histogram(d,bins = 'auto')
            d = d[:len(d)-1-len(np.where(d>=hd[1][len(hd[1])-2])[0])]
            MeanBC2noPers = np.mean(d0)
            SDBC2noPers = np.std(d0)
            SKBC2noPers = skew(d0)
            KurtBC2noPers = kurtosis(d0)
            MeanDC2noPers = np.mean(d1)
            SDDC2noPers = np.std(d1)
            SKDC2noPers = skew(d1)
            KurtDC2noPers = kurtosis(d1)
            SUMPersC2noPers = sum(d)
            MeanPersC2noPers = np.mean(d)
            SDPersC2noPers = np.std(d)
            SKPersC2noPers = skew(d)
            KurtPersC2noPers = kurtosis(d)
        elif (N2==1):
            MeanBC2 = diagrams[2][:,0]
            MeanDC2 = diagrams[2][:,1]
            SUMpersC2 = diagrams[2][:,1]-diagrams[2][:,0]
            SDBC2 = SDDC2 = SDPersC2 = SKPersC2 = SKBC2 = KurtBC2 = SKDC2 = KurtDC2 = KurtpersC2 = 1000
            MeanpersC2 = diagrams[2][:,1]-diagrams[2][:,0]
            MeanBC2noPers = SDBC2noPers = SKBC2noPers = KurtBC2noPers = MeanDC2noPers = SDDC2noPers = SKDC2noPers = 1000
            KurtDC2noPers = SUMPersC2noPers = MeanPersC2noPers = SDPersC2noPers = SKPersC2noPers = KurtPersC2noPers = 1000
    
    
    
    
    #save the features as an array
    #this first one is with dimension 2 features
    #ECGFeat = np.array([N0, MeanDC0, SDDC0, SUMpersC0, SKDC0, KurtDC0, MeanBC1, MeanDC1, SDBC1, SDDC1, SUMpersC1, SKPersC1,N1, SDPersC1, SKBC1, KurtBC1,SKDC1,KurtDC1,MeanpersC1,KurtpersC1,  N2, SDPersC2,MeanBC2, MeanDC2, SDBC2, SDDC2,SKPersC2, SUMpersC2,SKBC2, KurtBC2,SKDC2,KurtDC2,MeanpersC2,KurtpersC2,MeanDC0noPers, SDDC0noPers,SKDC0noPers,KurtDC0noPers,MeanBC1noPers,SDBC1noPers,SKBC1noPers,KurtBC1noPers,MeanDC1noPers,SDDC1noPers,SKDC1noPers,KurtDC1noPers,SUMPersC1noPers,MeanPersC1noPers,SDPersC1noPers,SKPersC1noPers, KurtPersC1noPers,MeanBC2noPers,SDBC2noPers,SKBC2noPers,KurtBC2noPers,MeanDC2noPers,SDDC2noPers,SKDC2noPers,KurtDC2noPers,SUMPersC2noPers,MeanPersC2noPers,SDPersC2noPers,SKPersC2noPers, KurtPersC2noPers])
    
    #This one does not have dimension 2 features
    ECGFeat = np.array([N0, MeanDC0, SDDC0, SUMpersC0, SKDC0, KurtDC0, MeanBC1, MeanDC1, SDBC1, SDDC1, SUMpersC1, SKPersC1,N1, SDPersC1, SKBC1, KurtBC1,SKDC1,KurtDC1,MeanpersC1,KurtpersC1,MeanDC0noPers, SDDC0noPers,SKDC0noPers,KurtDC0noPers,MeanBC1noPers,SDBC1noPers,SKBC1noPers,KurtBC1noPers,MeanDC1noPers,SDDC1noPers,SKDC1noPers,KurtDC1noPers,SUMPersC1noPers,MeanPersC1noPers,SDPersC1noPers,SKPersC1noPers, KurtPersC1noPers,
                       pers_entropy[0],pers_entropy[1]])
    return ECGFeat 


#This is a new TDA feature
def persland(diagram):
    diagram=sorted(diagram, key=lambda t: (t[0], -t[1]))
    diagram=np.array(diagram)
    k = 0
    L = []
    Peaks = []
    Valleys = []
    while len(diagram) != 0:
        i = 0
        p = 0
        pair = diagram[i]
        diagram = np.delete(diagram, i, axis = 0)
        L.append(np.array([[-math.inf,0], [pair[0],0], [0.5*(pair[0] + pair[1]), 0.5*(pair[1] - pair[0])]]))
        Peaks.append(np.array([[0.5*(pair[0] + pair[1]), 0.5*(pair[1] - pair[0])]]))
        Valleys.append(np.array([[pair[0],0]]))
        while all(L[k][-1] != np.array([math.inf, 0])):
            if len(diagram) == 0 or p==len(diagram):
                L[k] = np.append(L[k], np.array([[pair[1], 0]]), axis=0)
                #L[k] = np.append(L[k], np.array([[math.inf, 0]]), axis=0)
                break
            ran = range(p, len(diagram))
            if pair[1] >= max(diagram[:,1][ran]):
                L[k] = np.append(L[k], np.array([[pair[1], 0]]), axis=0)
                #L[k] = np.append(L[k], np.array([[math.inf, 0]]), axis=0)
            else:
                u = np.where(diagram[:,1] > pair[1])[0]
                j = min(u[np.where(u >= p)[0]])
                pair1 = diagram[j]
                diagram = np.delete(diagram, j, axis = 0)
                p = j
                if pair1[0] > pair[1]:
                    L[k] = np.append(L[k], np.array([[pair[1], 0]]), axis = 0)
                    Valleys[k] = np.append(Valleys[k], np.array([[pair[1],0], [pair1[0],0]]), axis = 0)
                if pair1[0] >= pair[1]:
                    L[k] = np.append(L[k], np.array([[pair1[0], 0]]), axis = 0)
                else:
                    b_1 = 0.5*(pair1[0] + pair[1])
                    d_1 = 0.5*(pair[1] - pair1[0])
                    L[k] = np.append(L[k], np.array([[b_1, d_1]]), axis=0)
                    diagram = np.append(diagram, np.array([[pair1[0],pair[1]]]), axis = 0)
                    ran = range(p, len(diagram))
                    diagram[ran] = sorted(diagram[ran], key=lambda t: (t[0], -t[1]))
                    Valleys[k] = np.append(Valleys[k], np.array([[b_1,d_1]]), axis = 0)
                b_2 = 0.5*(pair1[0] + pair1[1])
                d_2 = 0.5*(pair1[1] - pair1[0])
                L[k] = np.append(L[k], np.array([[b_2,d_2]]), axis = 0)
                Peaks[k] = np.append(Peaks[k], np.array([[b_2,d_2]]), axis = 0)
                pair = pair1
        Valleys[k] = np.append(Valleys[k], np.array([[pair[1], 0]]), axis = 0)
        k+=1
    #Features from the Persistence Landscapes
    
    Order = k #Maximum Number of Layers
    Num_Peaks = np.zeros(k)
    Num_Valls = np.zeros(k)

    for i in range(k):
        Num_Peaks[i] = len(Peaks[i]) #Number of Peaks Per Layer
        Num_Valls[i] = len(Valleys[i]) #Number of Valleys Per Layer
    if len(Num_Peaks)>1:
        Mean_Peaks = np.mean(Num_Peaks) #Mean of Peaks
        STD_Peaks = np.std(Num_Peaks) #Standard Deviation of Peaks
        Skew_Peaks = skew(np.asarray(Num_Peaks)) #Skewness of Peaks
        Kurt_Peaks = kurtosis(np.asarray(Num_Peaks)) #Kurtosis of Peaks
    else:
        Mean_Peaks = 0
        STD_Peaks = 0
        Skew_Peaks = 0
        Kurt_Peaks = 0
    if len(Num_Valls)>1:
        Mean_Valls = np.mean(Num_Valls) #Mean of Valleys     
        STD_Valls = np.std(Num_Valls) #Standard Deviation of Valleys
        Skew_Valls = skew(np.asarray(Num_Valls)) #Skewness Valleys
        Kurt_Valls = kurtosis(np.asarray(Num_Valls)) #Kurtosis of Peaks
    else:
        Mean_Valls = 0
        STD_Valls = 0
        Skew_Valls = 0
        Kurt_Valls = 0
    
    Areas = np.zeros(k)
    if k>1:
        Rel_Areas = np.zeros(k-1)
        for i in range(k):
            q = L[i]
            for j in range(1,len(q) - 1):
                b = 0.5*(q[j][1] + q[j+1][1])*(q[j+1][0] - q[j][0])
                Areas[i] = Areas[i] + b
        for i in range(0,k-2):
            Rel_Areas[i] = (Areas[i+1]-Areas[i+2])/Areas[1]
        Rel_Areas[k-2]=Areas[k-1]/Areas[1]
    else:
        Rel_Areas = np.array([1])
    
    Mean_Areas = np.mean(Rel_Areas)    
    STD_Areas = np.std(Rel_Areas)
    Skew_Areas = skew(np.asarray(Rel_Areas))
    Kurt_Areas = kurtosis(np.asarray(Rel_Areas)) 
    
    ECGFeat_Land = np.array([Order, Mean_Peaks, STD_Peaks, Skew_Peaks, Kurt_Peaks, Mean_Valls, STD_Valls, Skew_Valls, Kurt_Valls, Mean_Areas, STD_Areas, Skew_Areas, Kurt_Areas])
    
    return ECGFeat_Land




def get_12ECG_features(data, header_data,lead):
    tmp_hea = header_data.split('\n')[0]
    tmp_hea = tmp_hea.split(' ') 
    ptID = tmp_hea[0]
    num_leads = int(tmp_hea[1])
    sample_Fs= int(tmp_hea[2])
    gain_lead = np.zeros(num_leads)
    
    for ii in range(num_leads):
        tmp_hea = header_data.split('\n')[ii+1].split(' ')
        gain_lead[ii] = int(tmp_hea[2].split('/')[0])
    # for testing, we included the mean age of 57 if the age is a NaN
    # This value will change as more data is being released

#   We are only using data from lead1
    peaks,idx = detect_peaks(data[lead],sample_Fs,gain_lead[lead])
    if len(peaks)==0 or len(idx)==0:
        peaks,idx = detect_peaks(data[0],sample_Fs,gain_lead[0])
        mean_RR = np.mean(idx/sample_Fs*1000)
        mean_Peaks = np.mean(peaks*gain_lead[0])

    #   median
        median_RR = np.median(idx/sample_Fs*1000)
        median_Peaks = np.median(peaks*gain_lead[0])

    #   standard deviation
        std_RR = np.std(idx/sample_Fs*1000)
        std_Peaks = np.std(peaks*gain_lead[0])

    #   variance
        var_RR = stats.tvar(idx/sample_Fs*1000)
        var_Peaks = stats.tvar(peaks*gain_lead[0])

    #   Skewness
        skew_RR = stats.skew(idx/sample_Fs*1000)
        skew_Peaks = stats.skew(peaks*gain_lead[0])

    #   Kurtosis
        kurt_RR = stats.kurtosis(idx/sample_Fs*1000)
        kurt_Peaks = stats.kurtosis(peaks*gain_lead[0])
    else:
        #   mean
        mean_RR = np.mean(idx/sample_Fs*1000)
        mean_Peaks = np.mean(peaks*gain_lead[lead])

    #   median
        median_RR = np.median(idx/sample_Fs*1000)
        median_Peaks = np.median(peaks*gain_lead[lead])

    #   standard deviation
        std_RR = np.std(idx/sample_Fs*1000)
        std_Peaks = np.std(peaks*gain_lead[lead])

    #   variance
        var_RR = stats.tvar(idx/sample_Fs*1000)
        var_Peaks = stats.tvar(peaks*gain_lead[lead])

    #   Skewness
        skew_RR = stats.skew(idx/sample_Fs*1000)
        skew_Peaks = stats.skew(peaks*gain_lead[lead])

    #   Kurtosis
        kurt_RR = stats.kurtosis(idx/sample_Fs*1000)
        kurt_Peaks = stats.kurtosis(peaks*gain_lead[lead])

    features = np.hstack([mean_RR,mean_Peaks,median_RR,median_Peaks,std_RR,std_Peaks,var_RR,var_Peaks,skew_RR,skew_Peaks,kurt_RR,kurt_Peaks])

  
    return features


"""

#This is the code that runs all the extraction
def get_12ECG_features(data, header_data):
#    tmp_hea = header_data[0].split(' ')
    tmp_hea = header_data.split('\n')[0]
    tmp_hea = tmp_hea.split(' ') 
#    print(tmp_hea)
    ptID = tmp_hea[0]
    num_leads = int(tmp_hea[1])
    sample_Fs= int(tmp_hea[2])
    gain_lead = np.zeros(num_leads)

    for ii in range(num_leads):
        tmp_hea = header_data.split('\n')[ii+1].split(' ')

    # for testing, we included the mean age of 57 if the age is a NaN
    # This value will change as more data is being released
    for iline in header_data:
        if iline.startswith('#Age'):
            tmp_age = iline.split(': ')[1].strip()
            age = int(tmp_age if tmp_age != 'NaN' else 57)
        elif iline.startswith('#Sex'):
            tmp_sex = iline.split(': ')[1]
            if tmp_sex.strip()=='Female':
                sex =1
            else:
                sex=0
    #Extract RR Features
    RRtemp = np.array([]) 
    for lead in np.arange(data.shape[0]):
        RRtemp = np.append(RRtemp,get_12ECG_features_original(data, header_data,lead))
    return RRtemp
    
"""